## TODO - Fix Weekday situation
from elasticsearch import Elasticsearch, helpers
from geopy.geocoders import Nominatim
from time import strftime, localtime
import contextlib
import sys

_index_tweet_status = False
_index_follow_status = False
_index_user_status = False
_is_near_def = False
_is_location_def = False
_near = {}
_location = {}

geolocator = Nominatim(user_agent="twint-1.2")

class RecycleObject(object):
    def write(self, junk): pass
    def flush(self): pass

def getLocation(place, **options):
    location = geolocator.geocode(place,timeout=1000)
    if location:
        if options.get("near"):
            global _near
            _near = {"lat": location.latitude, "lon": location.longitude}
            return True
        elif options.get("location"):
            global _location
            _location = {"lat": location.latitude, "lon": location.longitude}
            return True
        return {"lat": location.latitude, "lon": location.longitude}
    else:
        return {}

def handleIndexResponse(response):
    try:
        if response["status"] == 400:
            return True
    except KeyError:
        pass
    if response["acknowledged"]:
        print("[+] Index \"" + response["index"] + "\" created!")
    else:
        print("[x] error index creation :: storage.elasticsearch.handleIndexCreation")
    if response["shards_acknowledged"]:
        print("[+] Shards acknowledged, everything is ready to be used!")
        return True
    else:
        print("[x] error with shards :: storage.elasticsearch.HandleIndexCreation")
        return False

def createIndex(config, instance, **scope):
    if scope.get("scope") == "tweet":
        tweets_body = {
                "mappings": {
                    "properties": {
                        "id": {"type": "long"},
                        "conversation_id": {"type": "long"},
                        "created_at": {"type": "long"},
                        "date": {"type": "date", "format": "yyyy-MM-dd HH:mm:ss"},
                        "timezone": {"type": "keyword"},
                        "place": {"type": "keyword"},
                        "location": {"type": "keyword"},
                        "tweet": {"type": "text"},
                        "hashtags": {"type": "keyword", "normalizer": "hashtag_normalizer"},
                        "cashtags": {"type": "keyword", "normalizer": "hashtag_normalizer"},
                        "user_id_str": {"type": "keyword"},
                        "username": {"type": "keyword", "normalizer": "hashtag_normalizer"},
                        "name": {"type": "text"},
                        "profile_image_url": {"type": "text"},
                        "day": {"type": "integer"},
                        "hour": {"type": "integer"},
                        "link": {"type": "text"},
                        "retweet": {"type": "text"},
                        "essid": {"type": "keyword"},
                        "nlikes": {"type": "integer"},
                        "nreplies": {"type": "integer"},
                        "nretweets": {"type": "integer"},
                        "quote_url": {"type": "text"},
                        "video": {"type":"integer"},
                        "search": {"type": "text"},
                        "near": {"type": "text"},
                        "geo_near": {"type": "geo_point"},
                        "geo_tweet": {"type": "geo_point"},
                        "photos": {"type": "text"},
                        "user_rt_id": {"type": "keyword"},
                        "mentions": {"type": "keyword", "normalizer": "hashtag_normalizer"},
                        "source": {"type": "keyword"},
                        "user_rt": {"type": "keyword"},
                        "retweet_id": {"type": "keyword"},
                        "reply_to": {
                            "type": "nested",
                            "properties": {
                                "user_id": {"type": "keyword"},
                                "username": {"type": "keyword"}
                            }
                        },
                        "retweet_date": {"type": "date", "format": "yyyy-MM-dd HH:mm:ss"},
                        "urls": {"type": "keyword"},
                        "translate": {"type": "text"},
                        "trans_src": {"type": "keyword"},
                        "trans_dest": {"type": "keyword"},
                        }
                    },
                    "settings": {
                        "number_of_shards": 1,
                        "analysis": {
                            "normalizer": {
                                "hashtag_normalizer": {
                                    "type": "custom",
                                    "char_filter": [],
                                    "filter": ["lowercase", "asciifolding"]
                                }
                            }
                        }
                    }
                }
        with nostdout():
            resp = instance.indices.create(index=config.Index_tweets, body=tweets_body, ignore=400)
        return handleIndexResponse(resp)
    elif scope.get("scope") == "follow":
        follow_body = {
                "mappings": {
                    "properties": {
                        "user": {"type": "keyword"},
                        "follow": {"type": "keyword"},
                        "essid": {"type": "keyword"}
                        }
                    },
                    "settings": {
                        "number_of_shards": 1
                    }
                }
        with nostdout():
            resp = instance.indices.create(index=config.Index_follow, body=follow_body, ignore=400)
        return handleIndexResponse(resp)
    elif scope.get("scope") == "user":
        user_body = {
                "mappings": {
                    "properties": {
                        "id": {"type": "keyword"},
                        "name": {"type": "keyword"},
                        "username": {"type": "keyword"},
                        "bio": {"type": "text"},
                        "location": {"type": "keyword"},
                        "url": {"type": "text"},
                        "join_datetime": {"type": "date", "format": "yyyy-MM-dd HH:mm:ss"},
                        "tweets": {"type": "integer"},
                        "following": {"type": "integer"},
                        "followers": {"type": "integer"},
                        "likes": {"type": "integer"},
                        "media": {"type": "integer"},
                        "private": {"type": "integer"},
                        "verified": {"type": "integer"},
                        "avatar": {"type": "text"},
                        "background_image": {"type": "text"},
                        "session": {"type": "keyword"},
                        "geo_user": {"type": "geo_point"}
                        }
                    },
                    "settings": {
                        "number_of_shards": 1
                    }
                }
        with nostdout():
            resp = instance.indices.create(index=config.Index_users, body=user_body, ignore=400)
        return handleIndexResponse(resp)
    else:
        print("[x] error index pre-creation :: storage.elasticsearch.createIndex")
        return False

@contextlib.contextmanager
def nostdout():
    savestdout = sys.stdout
    sys.stdout = RecycleObject()
    yield
    sys.stdout = savestdout

def weekday(day):
    weekdays = {
            "Monday": 1,
            "Tuesday": 2,
            "Wednesday": 3,
            "Thursday": 4,
            "Friday": 5,
            "Saturday": 6,
            "Sunday": 7,
            }

    return weekdays[day]

def hour(datetime):
    return strftime("%H", localtime(datetime))

def Tweet(Tweet, config):
    global _index_tweet_status
    global _is_near_def
    weekdays = {
            "Monday": 1,
            "Tuesday": 2,
            "Wednesday": 3,
            "Thursday": 4,
            "Friday": 5,
            "Saturday": 6,
            "Sunday": 7,
            }
    day = weekdays[strftime("%A", localtime(Tweet.datetime/1000))]

    actions = []

    dt = f"{Tweet.datestamp} {Tweet.timestamp}"

    j_data = {
            "_index": config.Index_tweets,
            "_id": str(Tweet.id) + "_raw_" + config.Essid,
            "_source": {
                "id": str(Tweet.id),
                "conversation_id": Tweet.conversation_id,
                "created_at": Tweet.datetime,
                "date": dt,
                "timezone": Tweet.timezone,
                "place": Tweet.place,
                "tweet": Tweet.tweet,
                "hashtags": Tweet.hashtags,
                "cashtags": Tweet.cashtags,
                "user_id_str": Tweet.user_id_str,
                "username": Tweet.username,
                "name": Tweet.name,
                "day": day,
                "hour": hour(Tweet.datetime/1000),
                "link": Tweet.link,
                "retweet": Tweet.retweet,
                "essid": config.Essid,
                "nlikes": int(Tweet.likes_count),
                "nreplies": int(Tweet.replies_count),
                "nretweets": int(Tweet.retweets_count),
                "quote_url": Tweet.quote_url,
                "video": Tweet.video,
                "search": str(config.Search),
                "near": config.Near
                }
            }
    if Tweet.retweet:
        j_data["_source"].update({"user_rt_id": Tweet.user_rt_id})
        j_data["_source"].update({"user_rt": Tweet.user_rt})
        j_data["_source"].update({"retweet_id": Tweet.retweet_id})
        j_data["_source"].update({"retweet_date": Tweet.retweet_date})
    if Tweet.reply_to:
        j_data["_source"].update({"reply_to": Tweet.reply_to})
    if Tweet.photos:
        _photos = []
        for photo in Tweet.photos:
            _photos.append(photo)
        j_data["_source"].update({"photos": _photos})
    if Tweet.mentions:
        _mentions = []
        for mention in Tweet.mentions:
            _mentions.append(mention)
        j_data["_source"].update({"mentions": _mentions})
    if Tweet.urls:
        _urls = []
        for url in Tweet.urls:
            _urls.append(url)
        j_data["_source"].update({"urls": _urls})
    if config.Near or config.Geo:
        if not _is_near_def:
            __geo = ""
            __near = ""
            if config.Geo:
                __geo = config.Geo
            if config.Near:
                __near = config.Near
            _is_near_def = getLocation(__near + __geo, near=True)
        if _near:
            j_data["_source"].update({"geo_near": _near})
    if Tweet.place:
        _t_place = getLocation(Tweet.place)
        if _t_place:
            j_data["_source"].update({"geo_tweet": getLocation(Tweet.place)})
    if Tweet.source:
        j_data["_source"].update({"source": Tweet.Source})
    if config.Translate:
        j_data["_source"].update({"translate": Tweet.translate})        
        j_data["_source"].update({"trans_src": Tweet.trans_src})
        j_data["_source"].update({"trans_dest": Tweet.trans_dest})

    actions.append(j_data)

    es = Elasticsearch(config.Elasticsearch, verify_certs=config.Skip_certs)
    if not _index_tweet_status:
        _index_tweet_status = createIndex(config, es, scope="tweet")
    with nostdout():
        helpers.bulk(es, actions, chunk_size=2000, request_timeout=200)
    actions = []

def Follow(user, config):
    global _index_follow_status
    actions = []

    if config.Following:
        _user = config.Username
        _follow = user
    else:
        _user = user
        _follow = config.Username
    j_data = {
            "_index": config.Index_follow,
            "_id": _user + "_" + _follow + "_" + config.Essid,
            "_source": {
                "user": _user,
                "follow": _follow,
                "essid": config.Essid
                }
            }
    actions.append(j_data)

    es = Elasticsearch(config.Elasticsearch, verify_certs=config.Skip_certs)
    if not _index_follow_status:
        _index_follow_status = createIndex(config, es, scope="follow")
    with nostdout():
        helpers.bulk(es, actions, chunk_size=2000, request_timeout=200)
    actions = []

def UserProfile(user, config):
    global _index_user_status
    global _is_location_def
    actions = []

    j_data = {
            "_index": config.Index_users,
            "_id": user.id + "_" + user.join_date + "_" + user.join_time + "_" + config.Essid,
            "_source": {
                "id": user.id,
                "name": user.name,
                "username": user.username,
                "bio": user.bio,
                "location": user.location,
                "url": user.url,
                "join_datetime": user.join_date + " " + user.join_time,
                "tweets": user.tweets,
                "following": user.following,
                "followers": user.followers,
                "likes": user.likes,
                "media": user.media_count,
                "private": user.is_private,
                "verified": user.is_verified,
                "avatar": user.avatar,
                "background_image": user.background_image,
                "session": config.Essid
                }
            }
    if config.Location:
        if not _is_location_def:
            _is_location_def = getLocation(user.location, location=True)
        if _location:
            j_data["_source"].update({"geo_user": _location})
    actions.append(j_data)

    es = Elasticsearch(config.Elasticsearch, verify_certs=config.Skip_certs)
    if not _index_user_status:
        _index_user_status = createIndex(config, es, scope="user")
    with nostdout():
        helpers.bulk(es, actions, chunk_size=2000, request_timeout=200)
    actions = []



from rake_nltk import Rake
import numpy as np
import math
import nltk
from nltk import word_tokenize
import string
from nltk.stem import WordNetLemmatizer

test = True

# download/update POS tag list and tag each token
nltk.download('averaged_perceptron_tagger')
# download/update lemmatizer english set
nltk.download('wordnet')


def extract(text_for_extract: str) -> set:
    return extract_v2(text_for_extract).union(extract_v1(text_for_extract))


# function to pre-process the text
def clean(text):
    text = text.lower()
    printable = set(string.printable)
    text = filter(lambda x: x in printable, text)
    text = "".join(list(text))
    return text


def lemetize(POS_tag):
    wordnet_lemmatizer = WordNetLemmatizer()

    adjective_tags = ['JJ', 'JJR', 'JJS']
    lemmatized_text = []

    for word in POS_tag:
        if word[1] in adjective_tags:
            lemmatized_text.append(str(wordnet_lemmatizer.lemmatize(word[0], pos="a")))
        else:
            lemmatized_text.append(str(wordnet_lemmatizer.lemmatize(word[0])))  # default POS = noun

    # print("Text tokens after lemmatization of adjectives and nouns: \n")
    # print(lemmatized_text)

    return lemmatized_text


def extract_v1(txt: str, lim: int = 25) -> set:
    # Uses stopwords for english from NLTK, and all puntuation characters by
    # default
    r = Rake()
    # Extraction given the text
    r.extract_keywords_from_text(txt)

    # Extraction given the list of strings where each string is a sentence.
    # sentences = nltk.tokenize.sent_tokenize(text)
    # r.extract_keywords_from_sentences(sentences)

    # To get keyword phrases ranked highest to lowest.
    # print(r.get_ranked_phrases())

    # To get keyword phrases ranked highest to lowest with scores.
    # print(r.get_ranked_phrases_with_scores())
    ranked_phrases = r.get_ranked_phrases()
    res = []

    for phrase in ranked_phrases:
        text = word_tokenize(phrase)
        POS_tag = nltk.pos_tag(text)

        res.append(" ".join(lemetize(POS_tag), ))

    return set(res[:(lim if len(res) >= lim else len(res))])


def extract_v2(txt: str, lim: int = 50) -> set:
    # algorithm from https://github.com/JRC1995/TextRank-Keyword-Extraction/blob/master/TextRank.ipynb

    # nltk.download('punkt')
    Cleaned_text = clean(txt)
    # print(Cleaned_text)
    text = word_tokenize(Cleaned_text)

    #print("Tokenized Text: \n")
    #print(text)

    # tag each token
    POS_tag = nltk.pos_tag(text)

    #print("Tokenized Text with POS tags: \n")
    #print(POS_tag)


    lemmatized_text = lemetize(POS_tag)

    # tag each token
    POS_tag = nltk.pos_tag(lemmatized_text)

    #print("Lemmatized text with POS tags: \n")
    #print(POS_tag)

    stopwords = []

    wanted_POS = ['CD', 'NN', 'NNS', 'NNP', 'NNPS', 'JJ', 'JJR', 'JJS', 'VBG', 'FW']

    for word in POS_tag:
        if word[1] not in wanted_POS:
            stopwords.append(word[0])

    punctuations = list(str(string.punctuation))

    stopwords = stopwords + punctuations
    stopword_file = open("stopwords.txt", "r")
    # Source = https://www.ranks.nl/stopwords

    lots_of_stopwords = []

    for line in stopword_file.readlines():
        lots_of_stopwords.append(str(line.strip()))

    stopwords_plus = stopwords + lots_of_stopwords
    stopwords_plus = set(stopwords_plus)
    processed_text = []
    for word in lemmatized_text:
        if word not in stopwords_plus:
            processed_text.append(word)
    #print(processed_text)
    vocabulary = list(set(processed_text))
    #print(vocabulary)

    vocab_len = len(vocabulary)

    weighted_edge = np.zeros((vocab_len, vocab_len), dtype=np.float32)

    score = np.zeros((vocab_len), dtype=np.float32)
    window_size = 3
    covered_coocurrences = []

    for i in range(0, vocab_len):
        score[i] = 1
        for j in range(0, vocab_len):
            if j == i:
                weighted_edge[i][j] = 0
            else:
                for window_start in range(0, (len(processed_text) - window_size)):

                    window_end = window_start + window_size

                    window = processed_text[window_start:window_end]

                    if (vocabulary[i] in window) and (vocabulary[j] in window):

                        index_of_i = window_start + window.index(vocabulary[i])
                        index_of_j = window_start + window.index(vocabulary[j])

                        # index_of_x is the absolute position of the xth term in the window
                        # (counting from 0)
                        # in the processed_text

                        if [index_of_i, index_of_j] not in covered_coocurrences:
                            weighted_edge[i][j] += 1 / math.fabs(index_of_i - index_of_j)
                            covered_coocurrences.append([index_of_i, index_of_j])

    inout = np.zeros((vocab_len), dtype=np.float32)

    for i in range(0, vocab_len):
        for j in range(0, vocab_len):
            inout[i] += weighted_edge[i][j]

    MAX_ITERATIONS = 50
    d = 0.85
    threshold = 0.0001  # convergence threshold

    for iter in range(0, MAX_ITERATIONS):
        prev_score = np.copy(score)

        for i in range(0, vocab_len):

            summation = 0
            for j in range(0, vocab_len):
                if weighted_edge[i][j] != 0:
                    summation += (weighted_edge[i][j] / inout[j]) * score[j]

            score[i] = (1 - d) + d * (summation)

        if np.sum(np.fabs(prev_score - score)) <= threshold:  # convergence condition
            #print("Converging at iteration " + str(iter) + "....")
            break

    #for i in range(0, vocab_len):
    #   print("Score of " + vocabulary[i] + ": " + str(score[i]))

    phrases = []

    phrase = " "
    for word in lemmatized_text:

        if word in stopwords_plus:
            if phrase != " ":
                phrases.append(str(phrase).strip().split())
            phrase = " "
        elif word not in stopwords_plus:
            phrase += str(word)
            phrase += " "

    #print("Partitioned Phrases (Candidate Keyphrases): \n")
    #print(phrases)

    unique_phrases = []

    for phrase in phrases:
        if phrase not in unique_phrases:
            unique_phrases.append(phrase)

    #print("Unique Phrases (Candidate Keyphrases): \n")
    #print(unique_phrases)

    for word in vocabulary:
        # print word
        for phrase in unique_phrases:
            if (word in phrase) and ([word] in unique_phrases) and (len(phrase) > 1):
                # if len(phrase)>1 then the current phrase is multi-worded.
                # if the word in vocabulary is present in unique_phrases as a single-word-phrase
                # and at the same time present as a word within a multi-worded phrase,
                # then I will remove the single-word-phrase from the list.
                unique_phrases.remove([word])

    #print("Thinned Unique Phrases (Candidate Keyphrases): \n")
    #print(unique_phrases)

    phrase_scores = []
    keywords = []
    for phrase in unique_phrases:
        phrase_score = 0
        keyword = ''
        for word in phrase:
            keyword += str(word)
            keyword += " "
            phrase_score += score[vocabulary.index(word)]
        phrase_scores.append(phrase_score)
        keywords.append(keyword.strip())

    i = 0
    for keyword in keywords:
        # print("Keyword: '" + str(keyword) + "', Score: " + str(phrase_scores[i]))
        i += 1

    sorted_index = np.flip(np.argsort(phrase_scores), 0)

    keywords_num = lim if len(keywords) >= lim else len(keywords)

    # print("Keywords:\n")

    res = []
    for i in range(0, keywords_num):
        res.append(str(keywords[sorted_index[i]]))

    return set(res)


if test:
    file = open("text_example.txt", "r")
    text = file.read()
    print(extract(text))
    print(extract_v1(text))
    print(extract_v2(text))
    file.close()


""" InfluxDB Singleton Class """
from influxdb import InfluxDBClient

class InfluxDBLog:  
    singleton = None 
    client_influxDB = None 

    def __new__(cls, *args, **kwargs):  
        if not cls.singleton:  
            cls.singleton = object.__new__(InfluxDBLog)  
        return cls.singleton  

    def __init__(self):
        self.client_influxDB = InfluxDBClient("server.trollhunter.guru",
                                                8086,
                                                "instapy",
                                                "password",
                                                "instapy")


    def addEntry(self, measurement, tag_name, tag_value, field1_name, field1_value):
        if self.client_influxDB is not None:
            json_body = [
                    {
                        "measurement": measurement,
                        "tags": {
                            tag_name: tag_value
                        },
                        "fields": {
                            field1_name: field1_value,
                            
                        }
                    }]
            self.client_influxDB.write_points(json_body)

   


import pandas as pd
class User:
    def __init__(self,username):
        self.username = username
        self.info_df = None
        self.info_follow_df = None
        self.interaction_df = None
        self.user_id = None


    def set_info_to_df(self,user_obj):
        self.info_df = pd.DataFrame.from_dict([vars(user_obj)])

    def set_user_id(self, user_id):
        self.user_id = user_id

    def set_follower_df(self, follow):
        self.follower_df = follow

    def set_following_df(self, follow):
        self.following_df = follow

    def set_tweet_df(self,tweet):
        self.tweets_df = tweet

    def set_follow_df(self, follow, follower_id, following_id):
        if self.info_follow_df is None:
            self.info_follow_df = pd.DataFrame.copy(follow)
        else:
            self.info_follow_df = pd.concat([self.info_follow_df, follow], join="inner")
        self.set_interaction(follower_id, "follow", following_id)

    def set_interaction(self, user_a, interaction, usr_b):
        if self.interaction_df is None:
            self.interaction_df = pd.DataFrame(columns=["id", "follower", "interaction", "following"])
        self.interaction_df = self.interaction_df.append({
            "id": str(user_a) + "_" + str(usr_b),
            'follower': user_a,
            'interaction': interaction,
            'following': usr_b
        }, ignore_index=True)







import pandas as pd
import time
from elasticsearch import Elasticsearch
from elasticsearch import helpers

# Check for empty values

from TrollHunter.news_crawler.sitemap.sitemap import parse_sitemap


def safe_value(field_val):
    return field_val if not pd.isna(field_val) else "Other"


def filterKeys(document, headers):
    use_these_keys = ["Source"] + headers + ['Trust level']
    return {key: document[key] for key in use_these_keys }


def doc_generator(df, headers):
    df_iter = df.iterrows()
    for index, document in df_iter:
        try:
            yield {
                "_index": 'sitemaps',
                "_type": "_doc",
                "_id" : f"{document['loc']}",
                "_source": filterKeys(document, headers),
            }
        except StopIteration:
            return


def elastic_sitemap(sitemap, trust_levels, db_sitemap, host = "142.93.170.234", port = 9200, user = "elastic", password = "changeme", sort = None, influxdb = False):
    es = Elasticsearch(hosts=[{'host': host, 'port': port}], http_auth=(user, password) )
    dataframe = parse_sitemap(sitemap, trust_levels, db_sitemap, es, indexEs = "sitemaps", sort = sort, influxdb = influxdb, range_check=20)
    print(dataframe)
    if type(dataframe) == bool:
        return

    transform_none_lastmod(dataframe)
    length_dataframe = len(dataframe.index)
    if length_dataframe > 0:
        print(length_dataframe, " doc(s) will be put in ES")
        print(helpers.bulk(es, doc_generator(dataframe, sitemap[2])))
    else:
        print("No new value")


def transform_none_lastmod(pdResult: pd.DataFrame):
    for index, row in pdResult.iterrows():
        date = row[2].split(".")[0]
        timestamp = 0
        if date != "None":
            date_in = date[:19]
            timestamp = time.mktime(time.strptime(date_in, '%Y-%m-%dT%H:%M:%S'))

        row[2] = timestamp


def iterator(ar):
    for item in ar:
        try:
            yield item
        except StopIteration:
            return



from TrollHunter.news_crawler.database import get_sitemap_parent, get_all_sitemap, get_trust_levels
from TrollHunter.news_crawler.sitemap import elastic_sitemap
import time


def news_crawler():
    db_sitemap = {x[0]: x for x in get_all_sitemap()}
    trust_levels = {x[0]: x[1] for x in get_trust_levels()}
    for parent in get_sitemap_parent():
        elastic_sitemap(parent, trust_levels, db_sitemap, sort='loc', influxdb=True)


def scheduler_news():
    print('Start crawler/indexer sitemap')
    while True:
        start = time.time()
        try:
            news_crawler()
        except Exception as error:
            print(error)
        print('Sleeping')
        sleep = time.time() - start
        if sleep < 7200:
            time.sleep(7200 - sleep)



from bs4 import BeautifulSoup
from re import findall
from json import loads

import logging as logme

def Follow(response):
    logme.debug(__name__+':Follow')
    soup = BeautifulSoup(response, "html.parser")
    follow = soup.find_all("td", "info fifty screenname")
    cursor = soup.find_all("div", "w-button-more")
    try:
        cursor = findall(r'cursor=(.*?)">', str(cursor))[0]
    except IndexError:
        logme.critical(__name__+':Follow:IndexError')

    return follow, cursor

def Mobile(response):
    logme.debug(__name__+':Mobile')
    soup = BeautifulSoup(response, "html.parser")
    tweets = soup.find_all("span", "metadata")
    max_id = soup.find_all("div", "w-button-more")
    try:
        max_id = findall(r'max_id=(.*?)">', str(max_id))[0]
    except Exception as e:
        logme.critical(__name__+':Mobile:' + str(e))

    return tweets, max_id

def profile(response):
    logme.debug(__name__+':profile')
    json_response = loads(response)
    html = json_response["items_html"]
    soup = BeautifulSoup(html, "html.parser")
    feed = soup.find_all("div", "tweet")

    return feed, feed[-1]["data-item-id"]

def Json(response):
    logme.debug(__name__+':Json')
    json_response = loads(response)
    html = json_response["items_html"]
    soup = BeautifulSoup(html, "html.parser")
    feed = soup.find_all("div", "tweet")
    return feed, json_response["min_position"]



from datetime import datetime
from elasticsearch import Elasticsearch, helpers

import json
import pandas as pd


class Elastic:
    def __init__(self, host="142.93.170.234", port=9200, user="elastic", password="changeme"):
        self.es = Elasticsearch(hosts=[{'host': host, 'port': port}], http_auth=(user, password))

    def store_users(self, users):
        print("Store ", len(users), " user(s)")
        print(helpers.bulk(self.es, self.doc_from_df(users, users.columns, "twitter_user", "id")))

    def store_tweets(self, tweets):
        print("Store ", len(tweets.index), " tweet(s)")
        print(helpers.bulk(self.es, self.doc_from_df(tweets, tweets.columns, "twitter_tweet", "id")))

    def store_interaction(self, interactions):
        print("Store ", len(interactions.index), " interaction(s)")
        print(helpers.bulk(self.es, self.doc_from_df(interactions, interactions.columns, "twitter_interaction", "id")))

    @staticmethod
    def filter_keys(document, headers):
        return {key: document[key] for key in headers}

    def doc_from_df(self, df, headers, index, id_key):
        df_iter = df.iterrows()
        for _, document in df_iter:
            try:
                yield {
                    "_index": index,
                    "_type": "_doc",
                    "_id": f"{document[id_key]}",
                    "_source": self.filter_keys(document, headers),
                }
            except StopIteration:
                return



import logging as logme

class user:
    type = "user"

    def __init__(self):
        pass

def inf(ur, _type):
    logme.debug(__name__+':inf')
    try:
        group = ur.find("div", "user-actions btn-group not-following ")
        if group == None:
            group = ur.find("div", "user-actions btn-group not-following")
        if group == None:
            group = ur.find("div", "user-actions btn-group not-following protected")
    except Exception as e:
        print("Error: " + str(e))

    if _type == "id":
        ret = group["data-user-id"]
    elif _type == "name":
        ret = group["data-name"]
    elif _type == "username":
        ret = group["data-screen-name"]
    elif _type == "private":
        ret = group["data-protected"]
        if ret == 'true':
            ret = 1
        else:
            ret = 0

    return ret

def card(ur, _type):
    logme.debug(__name__+':card')
    if _type == "bio":
        try:
            ret = ur.find("p", "ProfileHeaderCard-bio u-dir").text.replace("\n", " ")
        except:
            ret = ""
    elif _type == "location":
        try:
            ret = ur.find("span", "ProfileHeaderCard-locationText u-dir").text
            ret = ret[15:].replace("\n", " ")[:-10]
        except:
            ret = ""
    elif _type == "url":
        try:
            ret = ur.find("span", "ProfileHeaderCard-urlText u-dir").find("a")["title"]
        except:
            ret = ""

    return ret

def join(ur):
    logme.debug(__name__+':join')
    jd = ur.find("span", "ProfileHeaderCard-joinDateText js-tooltip u-dir")["title"]
    return jd.split(" - ")

def convertToInt(x):
    logme.debug(__name__+':contertToInt')
    multDict = {
        "k" : 1000,
        "m" : 1000000,
        "b" : 1000000000,
    }
    try:
        if ',' in x:
            x = x.replace(',', '')
        y = int(x)
        return y
    except:
        pass

    try:
        y = float(str(x)[:-1])
        y = y * multDict[str(x)[-1:].lower()]
        return int(y)
    except:
        pass

    return 0

def stat(ur, _type):
    logme.debug(__name__+':stat')
    _class = f"ProfileNav-item ProfileNav-item--{_type}"
    stat = ur.find("li", _class)
    try :
        return int(stat.find("span", "ProfileNav-value")["data-count"])
    except AttributeError:
        return 0

def media(ur):
    logme.debug(__name__+':media')
    try:
        media_count = ur.find("a", "PhotoRail-headingWithCount js-nav").text.strip().split(" ")[0]
        return convertToInt(media_count)
    except:
        return 0

def verified(ur):
    logme.debug(__name__+':verified')
    try:
        is_verified = ur.find("span", "ProfileHeaderCard-badges").text
        if "Verified account" in is_verified:
            is_verified = 1
        else:
            is_verified = 0
    except:
        is_verified = 0

    return is_verified

def User(ur):
    logme.debug(__name__+':User')
    u = user()
    for img in ur.findAll("img", "Emoji Emoji--forText"):
        img.replaceWith(img["alt"])
    u.id = inf(ur, "id")
    u.name = inf(ur, "name")
    u.username = inf(ur, "username")
    u.bio = card(ur, "bio")
    u.location = card(ur, "location")
    u.url = card(ur, "url")
    u.join_date = join(ur)[1]
    u.join_time = join(ur)[0]
    u.tweets = stat(ur, "tweets is-active")
    u.following = stat(ur, "following")
    u.followers = stat(ur, "followers")
    u.likes = stat(ur, "favorites")
    u.media_count = media(ur)
    u.is_private = inf(ur, "private")
    u.is_verified = verified(ur)
    u.avatar = ur.find("img", "ProfileAvatar-image")["src"]
    u.background_image = ur.find('div',{'class':'ProfileCanopy-headerBg'}).find('img').get('src')
    return u



import sqlite3
import sys
import importlib

sys.path.append("../")
deMod = importlib.import_module("data_elastic")

# connect to the local DB
conn = sqlite3.connect('./trollhunter.db')
print("Opened database successfully")

# select all providers
cursor = conn.execute("SELECT id, url_provider, site_map_path from NEWS_PROVIDERS")

headers = ["loc", "lastmod", "image:loc", "news:keywords", "news:title"]

for row in cursor:
    site_map_url = row[1]+row[2]
    deMod.elastic_sitemap(site_map_url, headers, sort = "lastmod")

print("Operation done successfully")
conn.close()


from nltk.sentiment.vader import SentimentIntensityAnalyzer
from textblob import TextBlob
import pandas as pd


def get_sentiment_from_tweets(tweet):
    analyser = SentimentIntensityAnalyzer()
    sentiment_scores = []

    for t in tweet:
        score = analyser.polarity_scores(t)
        score['tweet'] = t
        sentiment_scores.append(score)

    sentiment_df = pd.DataFrame(sentiment_scores)

    return sentiment_df


def get_polarity(tweet):
    tab_result = {'tweet': [],
                  'polarity': []}

    for t in tweet:
        obj = TextBlob(t)
        polarity = obj.sentiment.polarity

        tab_result['tweet'].append(t)
        tab_result['polarity'].append(polarity)

    df_result = pd.DataFrame(tab_result, columns=['tweet', 'polarity'])

    return df_result


def get_subjectivity(tweet):
    tab_result = {'tweet': [],
                  'subjectivity': []}

    for t in tweet:
        obj = TextBlob(t)
        subjectivity = obj.sentiment.subjectivity

        tab_result['tweet'].append(t)
        tab_result['subjectivity'].append(subjectivity)

    df_result = pd.DataFrame(tab_result, columns=['tweet', 'subjectivity'])

    return df_result


if __name__ == '__main__':
    tweets = ["I'm so happy", "I surprised him", "It's a sad news", "I'm happy and sad at the same time"]
    sentiment = get_sentiment_from_tweets(tweets)
    polarity = get_polarity(tweets)
    sub = get_subjectivity(tweets)

    print("SENTIMENT\n", sentiment, "\n")
    print("POLARITY\n", polarity, "\n")
    print("SUBJECTIVITY\n", sub, "\n")



import unidecode

skip_charac = ["\"", "\'", "\n"]


class Tweet_obj:
    def __init__(self, tweet):
        self.datestamp = tweet.datestamp
        self.timestamp = tweet.timestamp
        self.date = tweet.datestamp + ":" + tweet.timestamp
        self.username = tweet.username
        self.name = tweet.name
        self.likes_count = tweet.likes_count
        self.retweets_count = tweet.retweets_count
        self.tweet = tweet.tweet

    def check_equal(self, tweet):
        return self.tweet_cleaner(self.tweet) == self.tweet_cleaner(tweet)

    def pretty_print(self):
        print("DATE : {} \n USER : {} \n TWEET : {}".format(self.date, self.username, self.tweet))

    @staticmethod
    def tweet_cleaner(tweet):
        result = unidecode.unidecode(tweet)
        for c in skip_charac:
            result = result.replace(c, '')

        return result.lower()



from __future__ import absolute_import
from celery import Celery
from celery.bin import worker
app = Celery('twitter-crawler',
             broker='pyamqp://guest@142.93.170.234',
             include=['TrollHunter.twitter_crawler.twint_api.request_twint'])
def run():
    worker_celery = worker.worker(app=app)
    option = {'loglevel': 'INFO'}
    worker_celery.run(**option)
    # app.start(["-A", "request", "worker", "--loglevel","info"])

if __name__ == '__main__':
    run()


#!/usr/bin/env python3
'''
Twint.py - Twitter Intelligence Tool (formerly known as Tweep).

See wiki on Github for in-depth details.
https://github.com/twintproject/twint/wiki

Licensed under MIT License
Copyright (c) 2018 The Twint Project  
'''
import sys
import os
import argparse

from . import run
from . import config
from . import storage

def error(_error, message):
    """ Print errors to stdout
    """
    print("[-] {}: {}".format(_error, message))
    sys.exit(0)

def check(args):
    """ Error checking
    """
    if args.username is not None or args.userlist or args.members_list:
        if args.verified:
            error("Contradicting Args",
                  "Please use --verified in combination with -s.")
        if args.userid:
            error("Contradicting Args",
                  "--userid and -u cannot be used together.")
        if args.all:
            error("Contradicting Args",
                  "--all and -u cannot be used together")
    elif args.search is None:
        if (args.geo or args.near) is None and not (args.all or args.userid):
            error("Error", "Please use at least -u, -s, -g or --near.")
    elif args.all and args.userid:
        error("Contradicting Args",
              "--all and --userid cannot be used together")
    if args.output is None:
        if args.csv:
            error("Error", "Please specify an output file (Example: -o file.csv).")
        elif args.json:
            error("Error", "Please specify an output file (Example: -o file.json).")

def loadUserList(ul, _type):
    """ Concatenate users
    """
    if os.path.exists(os.path.abspath(ul)):
        userlist = open(os.path.abspath(ul), "r").read().splitlines()
    else:
        userlist = ul.split(",")
    if _type == "search":
        un = ""
        for user in userlist:
            un += "%20OR%20from%3A" + user
        return un[15:]
    return userlist


def initialize(args):
    """ Set default values for config from args
    """
    c = config.Config()
    c.Username = args.username
    c.User_id = args.userid
    c.Search = args.search
    c.Geo = args.geo
    c.Location = args.location
    c.Near = args.near
    c.Lang = args.lang
    c.Output = args.output
    c.Elasticsearch = args.elasticsearch
    c.Year = args.year
    c.Since = args.since
    c.Until = args.until
    c.Email = args.email
    c.Phone = args.phone
    c.Verified = args.verified
    c.Store_csv = args.csv
    c.Store_json = args.json
    c.Show_hashtags = args.hashtags
    c.Show_cashtags = args.cashtags
    c.Limit = args.limit
    c.Count = args.count
    c.Stats = args.stats
    c.Database = args.database
    c.To = args.to
    c.All = args.all
    c.Essid = args.essid
    c.Format = args.format
    c.User_full = args.user_full
    c.Profile_full = args.profile_full
    c.Pandas_type = args.pandas_type
    c.Index_tweets = args.index_tweets
    c.Index_follow = args.index_follow
    c.Index_users = args.index_users
    c.Debug = args.debug
    c.Resume = args.resume
    c.Images = args.images
    c.Videos = args.videos
    c.Media = args.media
    c.Replies = args.replies
    c.Pandas_clean = args.pandas_clean
    c.Proxy_host = args.proxy_host
    c.Proxy_port = args.proxy_port
    c.Proxy_type = args.proxy_type
    c.Retweets = args.retweets
    c.Custom_query = args.custom_query
    c.Popular_tweets =  args.popular_tweets
    c.Skip_certs = args.skip_certs
    c.Hide_output = args.hide_output
    c.Native_retweets = args.native_retweets
    c.Min_likes = args.min_likes
    c.Min_retweets = args.min_retweets
    c.Min_replies = args.min_replies
    c.Links = args.links
    c.Source = args.source
    c.Members_list = args.members_list
    c.Filter_retweets = args.filter_retweets
    c.Translate = args.translate
    c.TranslateDest = args.translate_dest
    return c

def options():
    """ Parse arguments
    """
    ap = argparse.ArgumentParser(prog="twint",
                                 usage="python3 %(prog)s [options]",
                                 description="TWINT - An Advanced Twitter Scraping Tool.")
    ap.add_argument("-u", "--username", help="User's Tweets you want to scrape.")
    ap.add_argument("-s", "--search", help="Search for Tweets containing this word or phrase.")
    ap.add_argument("-g", "--geo", help="Search for geocoded Tweets.")
    ap.add_argument("--near", help="Near a specified city.")
    ap.add_argument("--location", help="Show user's location (Experimental).", action="store_true")
    ap.add_argument("-l", "--lang", help="Search for Tweets in a specific language.")
    ap.add_argument("-o", "--output", help="Save output to a file.")
    ap.add_argument("-es", "--elasticsearch", help="Index to Elasticsearch.")
    ap.add_argument("--year", help="Filter Tweets before specified year.")
    ap.add_argument("--since", help="Filter Tweets sent since date (Example: \"2017-12-27 20:30:15\" or 2017-12-27).",
                    metavar="DATE")
    ap.add_argument("--until", help="Filter Tweets sent until date (Example: \"2017-12-27 20:30:15\" or 2017-12-27).",
                    metavar="DATE")
    ap.add_argument("--email", help="Filter Tweets that might have email addresses", action="store_true")
    ap.add_argument("--phone", help="Filter Tweets that might have phone numbers", action="store_true")
    ap.add_argument("--verified", help="Display Tweets only from verified users (Use with -s).",
                    action="store_true")
    ap.add_argument("--csv", help="Write as .csv file.", action="store_true")
    ap.add_argument("--json", help="Write as .json file", action="store_true")
    ap.add_argument("--hashtags", help="Output hashtags in seperate column.", action="store_true")
    ap.add_argument("--cashtags", help="Output cashtags in seperate column.", action="store_true")
    ap.add_argument("--userid", help="Twitter user id.")
    ap.add_argument("--limit", help="Number of Tweets to pull (Increments of 20).")
    ap.add_argument("--count", help="Display number of Tweets scraped at the end of session.",
                    action="store_true")
    ap.add_argument("--stats", help="Show number of replies, retweets, and likes.",
                    action="store_true")
    ap.add_argument("-db", "--database", help="Store Tweets in a sqlite3 database.")
    ap.add_argument("--to", help="Search Tweets to a user.", metavar="USERNAME")
    ap.add_argument("--all", help="Search all Tweets associated with a user.", metavar="USERNAME")
    ap.add_argument("--followers", help="Scrape a person's followers.", action="store_true")
    ap.add_argument("--following", help="Scrape a person's follows", action="store_true")
    ap.add_argument("--favorites", help="Scrape Tweets a user has liked.", action="store_true")
    ap.add_argument("--proxy-type", help="Socks5, HTTP, etc.")
    ap.add_argument("--proxy-host", help="Proxy hostname or IP.")
    ap.add_argument("--proxy-port", help="The port of the proxy server.")
    ap.add_argument("--essid",
                    help="Elasticsearch Session ID, use this to differentiate scraping sessions.",
                    nargs="?", default="")
    ap.add_argument("--userlist", help="Userlist from list or file.")
    ap.add_argument("--retweets",
                    help="Include user's Retweets (Warning: limited).",
                    action="store_true")
    ap.add_argument("--format", help="Custom output format (See wiki for details).")
    ap.add_argument("--user-full",
                    help="Collect all user information (Use with followers or following only).",
                    action="store_true")
    ap.add_argument("--profile-full",
                    help="Slow, but effective method of collecting a user's Tweets and RT.",
                    action="store_true")
    ap.add_argument("--translate",
                    help="Get tweets translated by Google Translate.",
                    action="store_true")
    ap.add_argument("--translate-dest", help="Translate tweet to language (ISO2).",
                    default="en")
    ap.add_argument("--store-pandas", help="Save Tweets in a DataFrame (Pandas) file.")
    ap.add_argument("--pandas-type",
                    help="Specify HDF5 or Pickle (HDF5 as default)", nargs="?", default="HDF5")
    ap.add_argument("-it", "--index-tweets",
                    help="Custom Elasticsearch Index name for Tweets.", nargs="?", default="twinttweets")
    ap.add_argument("-if", "--index-follow",
                    help="Custom Elasticsearch Index name for Follows.",
                    nargs="?", default="twintgraph")
    ap.add_argument("-iu", "--index-users", help="Custom Elasticsearch Index name for Users.",
                    nargs="?", default="twintuser")
    ap.add_argument("--debug",
                    help="Store information in debug logs", action="store_true")
    ap.add_argument("--resume", help="Resume from Tweet ID.", metavar="TWEET_ID")
    ap.add_argument("--videos", help="Display only Tweets with videos.", action="store_true")
    ap.add_argument("--images", help="Display only Tweets with images.", action="store_true")
    ap.add_argument("--media",
                    help="Display Tweets with only images or videos.", action="store_true")
    ap.add_argument("--replies", help="Display replies to a subject.", action="store_true")
    ap.add_argument("-pc", "--pandas-clean",
                    help="Automatically clean Pandas dataframe at every scrape.")
    ap.add_argument("-cq", "--custom-query", help="Custom search query.")
    ap.add_argument("-pt", "--popular-tweets", help="Scrape popular tweets instead of recent ones.", action="store_true")
    ap.add_argument("-sc", "--skip-certs", help="Skip certs verification, useful for SSC.", action="store_false")
    ap.add_argument("-ho", "--hide-output", help="Hide output, no tweets will be displayed.", action="store_true")
    ap.add_argument("-nr", "--native-retweets", help="Filter the results for retweets only.", action="store_true")
    ap.add_argument("--min-likes", help="Filter the tweets by minimum number of likes.")
    ap.add_argument("--min-retweets", help="Filter the tweets by minimum number of retweets.")
    ap.add_argument("--min-replies", help="Filter the tweets by minimum number of replies.")
    ap.add_argument("--links", help="Include or exclude tweets containing one o more links. If not specified"+
                    " you will get both tweets that might contain links or not.")
    ap.add_argument("--source", help="Filter the tweets for specific source client.")
    ap.add_argument("--members-list", help="Filter the tweets sent by users in a given list.")
    ap.add_argument("-fr", "--filter-retweets", help="Exclude retweets from the results.", action="store_true")
    args = ap.parse_args()

    return args

def main():
    """ Main
    """
    args = options()
    check(args)

    if args.pandas_clean:
        storage.panda.clean()

    c = initialize(args)

    if args.userlist:
        c.Query = loadUserList(args.userlist, "search")

    if args.pandas_clean:
        storage.panda.clean()

    if args.favorites:
        if args.userlist:
            _userlist = loadUserList(args.userlist, "favorites")
            for _user in _userlist:
                args.username = _user
                c = initialize(args)
                run.Favorites(c)
        else:
            run.Favorites(c)
    elif args.following:
        if args.userlist:
            _userlist = loadUserList(args.userlist, "following")
            for _user in _userlist:
                args.username = _user
                c = initialize(args)
                run.Following(c)
        else:
            run.Following(c)
    elif args.followers:
        if args.userlist:
            _userlist = loadUserList(args.userlist, "followers")
            for _user in _userlist:
                args.username = _user
                c = initialize(args)
                run.Followers(c)
        else:
            run.Followers(c)
    elif args.retweets or args.profile_full:
        if args.userlist:
            _userlist = loadUserList(args.userlist, "profile")
            for _user in _userlist:
                args.username = _user
                c = initialize(args)
                run.Profile(c)
        else:
            run.Profile(c)
    elif args.user_full:
        if args.userlist:
            _userlist = loadUserList(args.userlist, "userlist")
            for _user in _userlist:
                args.username = _user
                c = initialize(args)
                run.Lookup(c)
        else:
            run.Lookup(c)
    else:
        run.Search(c)

def run_as_command():
    version = ".".join(str(v) for v in sys.version_info[:2])
    if float(version) < 3.6:
        print("[-] TWINT requires Python version 3.6+.")
        sys.exit(0)

    main()



'''
TWINT - Twitter Intelligence Tool (formerly known as Tweep).

See wiki on Github for in-depth details.
https://github.com/twintproject/twint/wiki

Licensed under MIT License
Copyright (c) 2018 Cody Zacharias
'''
import logging, os

from .config import Config
from . import run

_levels = {
    'info': logging.INFO,
    'debug': logging.DEBUG
}

_level = os.getenv('TWINT_DEBUG', 'info')
_logLevel = _levels[_level]

if _level == "debug":
    logger = logging.getLogger()
    _output_fn = 'twint.log'
    logger.setLevel(_logLevel)
    formatter = logging.Formatter('%(levelname)s:%(asctime)s:%(name)s:%(message)s')
    fileHandler = logging.FileHandler(_output_fn)
    fileHandler.setLevel(_logLevel)
    fileHandler.setFormatter(formatter)
    logger.addHandler(fileHandler)



from datetime import datetime

from . import format
from .tweet import Tweet
from .user import User
from .storage import db, elasticsearch, write, panda

import logging as logme

follows_list = []
tweets_list = []
users_list = []

author_list = {''}
author_list.pop()

# used by Pandas
_follows_object = {}

def _clean_follow_list():
    logme.debug(__name__+':clean_follow_list')
    global _follows_object
    _follows_object = {}

def clean_lists():
    logme.debug(__name__+':clean_lists')
    global follows_list
    global tweets_list
    global users_list
    follows_list = []
    tweets_list = []
    users_list = []

def datecheck(datetimestamp, config):
    logme.debug(__name__+':datecheck')
    if config.Since and config.Until:
        logme.debug(__name__+':datecheck:dateRangeTrue')
        d = int(datetime.strptime(datetimestamp, "%Y-%m-%d %H:%M:%S").timestamp())
        s = int(datetime.strptime(config.Since, "%Y-%m-%d %H:%M:%S").timestamp())
        if d < s:
           return False
    logme.debug(__name__+':datecheck:dateRangeFalse')
    return True

def is_tweet(tw):
    try:
        tw["data-item-id"]
        logme.debug(__name__+':is_tweet:True')
        return True
    except:
        logme.critical(__name__+':is_tweet:False')
        return False

def _output(obj, output, config, **extra):
    logme.debug(__name__+':_output')
    if config.Lowercase:
        if isinstance(obj, str):
            logme.debug(__name__+':_output:Lowercase:username')
            obj = obj.lower()
        elif obj.__class__.__name__ == "user":
            logme.debug(__name__+':_output:Lowercase:user')
            pass
        elif obj.__class__.__name__ == "tweet":
            logme.debug(__name__+':_output:Lowercase:tweet')
            obj.username = obj.username.lower()
            author_list.update({obj.username})
            for i in range(len(obj.mentions)):
                obj.mentions[i] = obj.mentions[i].lower()
            for i in range(len(obj.hashtags)):
                obj.hashtags[i] = obj.hashtags[i].lower()
            for i in range(len(obj.cashtags)):
                obj.cashtags[i] = obj.cashtags[i].lower()
        else:
            logme.info('_output:Lowercase:hiddenTweetFound')
            print("[x] Hidden tweet found, account suspended due to violation of TOS")
            return
    if config.Output != None:
        if config.Store_csv:
            try:
                write.Csv(obj, config)
                logme.debug(__name__+':_output:CSV')
            except Exception as e:
                logme.critical(__name__+':_output:CSV:Error:' + str(e))
                print(str(e) + " [x] output._output")
        elif config.Store_json:
            write.Json(obj, config)
            logme.debug(__name__+':_output:JSON')
        else:
            write.Text(output, config.Output)
            logme.debug(__name__+':_output:Text')

    if config.Elasticsearch:
        logme.debug(__name__+':_output:Elasticsearch')
        print("", end=".", flush=True)
    else:
        if not config.Hide_output:
            try:
                print(output.replace('\n', ' '))
            except UnicodeEncodeError:
                logme.critical(__name__+':_output:UnicodeEncodeError')
                print("unicode error [x] output._output")

async def checkData(tweet, config, conn):
    logme.debug(__name__+':checkData')
    copyright = tweet.find("div", "StreamItemContent--withheld")
    if copyright is None and is_tweet(tweet):
        tweet = Tweet(tweet, config)

        if not tweet.datestamp:
            logme.critical(__name__+':checkData:hiddenTweetFound')
            print("[x] Hidden tweet found, account suspended due to violation of TOS")
            return

        if datecheck(tweet.datestamp + " " + tweet.timestamp, config):
            output = format.Tweet(config, tweet)

            if config.Database:
                logme.debug(__name__+':checkData:Database')
                db.tweets(conn, tweet, config)

            if config.Pandas:
                logme.debug(__name__+':checkData:Pandas')
                panda.update(tweet, config)

            if config.Store_object:
                logme.debug(__name__+':checkData:Store_object')
                if hasattr(config.Store_object_tweets_list, 'append'):
                    config.Store_object_tweets_list.append(tweet)
                else:
                    tweets_list.append(tweet)

            if config.Elasticsearch:
                logme.debug(__name__+':checkData:Elasticsearch')
                elasticsearch.Tweet(tweet, config)

            _output(tweet, output, config)
    else:
        logme.critical(__name__+':checkData:copyrightedTweet')

async def Tweets(tweets, config, conn, url=''):
    logme.debug(__name__+':Tweets')
    if config.Favorites or config.Profile_full or config.Location:
        logme.debug(__name__+':Tweets:fav+full+loc')
        for tw in tweets:
            if tw['data-item-id'] == url.split('?')[0].split('/')[-1]:
                await checkData(tw, config, conn)
    elif config.TwitterSearch:
        logme.debug(__name__+':Tweets:TwitterSearch')
        await checkData(tweets, config, conn)
    else:
        logme.debug(__name__+':Tweets:else')
        if int(tweets["data-user-id"]) == config.User_id or config.Retweets:
            await checkData(tweets, config, conn)

async def Users(u, config, conn):
    logme.debug(__name__+':User')
    global users_list

    user = User(u)
    output = format.User(config.Format, user)

    if config.Database:
        logme.debug(__name__+':User:Database')
        db.user(conn, config, user)

    if config.Elasticsearch:
        logme.debug(__name__+':User:Elasticsearch')
        _save_date = user.join_date
        _save_time = user.join_time
        user.join_date = str(datetime.strptime(user.join_date, "%d %b %Y")).split()[0]
        user.join_time = str(datetime.strptime(user.join_time, "%I:%M %p")).split()[1]
        elasticsearch.UserProfile(user, config)
        user.join_date = _save_date
        user.join_time = _save_time

    if config.Store_object:
        logme.debug(__name__+':User:Store_object')
        users_list.append(user) # twint.user.user
    
    if config.Pandas:
        logme.debug(__name__+':User:Pandas+user')
        panda.update(obj, config)

    _output(user, output, config)

async def Username(username, config, conn):
    logme.debug(__name__+':Username')
    global _follows_object
    global follows_list
    follow_var = config.Following*"following" + config.Followers*"followers"

    if config.Database:
        logme.debug(__name__+':Username:Database')
        db.follow(conn, config.Username, config.Followers, username)

    if config.Elasticsearch:
        logme.debug(__name__+':Username:Elasticsearch')
        elasticsearch.Follow(username, config)

    if config.Store_object:
        follows_list.append(username)

    if config.Pandas:
        logme.debug(__name__+':Username:object+pandas')
        try:
            _ = _follows_object[config.Username][follow_var]
        except KeyError:
            _follows_object.update({config.Username: {follow_var: []}})
        _follows_object[config.Username][follow_var].append(username)
        if config.Pandas_au:
            logme.debug(__name__+':Username:object+pandas+au')
            panda.update(_follows_object[config.Username], config)
    _output(username, username, config)



import sys
sys.path.append("../")
from TrollHunter.news_crawler.sitemap import parse_sitemap
import time
import pandas as pd

class SitemapInfo:
    def __init__(self, baseUrl, headers):
        self._headers = headers
        self._baseUrl = baseUrl
        self._pdResult = self._retriveUrlsByMap(self._baseUrl)
        if self._pdResult is not False:
            self._fetchAllUrlInfo()

    def _retriveUrlsByMap(self, map, baseUrl, headers):
        result = parse_sitemap(map, )
        # print(result)
        return result if result is not False else pd.DataFrame()

    def _fetchAllUrlInfo(self):
        for index, row in self._pdResult.iterrows():
            date = row[2].split(".")[0]
            timestamp = 0
            if date != "None":
                timestamp = time.mktime(time.strptime(date, '%Y-%m-%dT%H:%M:%S'))

            row[2] = timestamp

    def getResult(self) -> pd.DataFrame:
        return self._pdResult



# Always prefer setuptools over distutils
from setuptools import setup

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
setup(
    name='TrollHunter',


    version='0.2.10',
    description='TrollHunter',
    url="https://github.com/StanGirard/TrollHunter",
    long_description=long_description,
    long_description_content_type='text/markdown',
    license="GPL",
    classifiers=[
        'Programming Language :: Python :: 3',
        'Development Status :: 5 - Production/Stable',
    ],

    packages=["TrollHunter.twitter_crawler","TrollHunter.twitter_crawler.twint_api",
              "TrollHunter.twitter_crawler.twint.twint","TrollHunter.twitter_crawler.twint.twint.storage",
              "TrollHunter.loggers","TrollHunter.news_crawler.sitemap","TrollHunter.news_crawler.database", "TrollHunter.texto", "TrollHunter.news_crawler"],

    install_requires=[
        "aiodns==2.0.0",
        "aiofiles==0.4.0",
        "aiohttp==3.6.2",
        "aiohttp-socks==0.3.4",
        "amqp==2.5.2",
        "aniso8601==8.0.0",
        "async-timeout==3.0.1",
        "attrs==19.3.0",
        "beautifulsoup4==4.8.2",
        "billiard==3.6.3.0",
        "blinker==1.4",
        "cchardet==2.1.5",
        "celery==4.4.1",
        "certifi==2019.11.28",
        "cffi==1.14.0",
        "chardet==3.0.4",
        "Click==7.0",
        "elasticsearch==7.5.1",
        "fake-useragent==0.1.11",
        "Flask==1.1.1",
        "Flask-Jsonpify==1.5.0",
        "Flask-RESTful==0.3.8",
        "Flask-SQLAlchemy==2.4.1",
        "geographiclib==1.50",
        "geopy==1.21.0",
        "googletransx==2.4.2",
        "h11==0.9.0",
        "h2==3.2.0",
        "hpack==3.0.0",
        "hyperframe==5.2.0",
        "idna==2.9",
        "influxdb==5.2.3",
        "importlib-metadata==1.5.0",
        "itsdangerous==1.1.0",
        "Jinja2==2.11.1",
        "kombu==4.6.8",
        "MarkupSafe==1.1.1",
        "multidict==4.7.5",
        "numpy==1.18.1",
        "pandas==1.0.1",
        "priority==1.3.0",
        "psycopg2-binary==2.8.4",
        "pycares==3.1.1",
        "pycparser==2.19",
        "PySocks==1.7.1",
        "python-dateutil==2.8.1",
        "pytz==2019.3",
        "requests==2.23.0",
        "schedule==0.6.0",
        "six==1.14.0",
        "soupsieve==2.0",
        "SQLAlchemy==1.3.13",
        "toml==0.10.0",
        "typing-extensions==3.7.4.1",
        "Unidecode==1.1.1",
        "urllib3==1.25.8",
        "vine==1.3.0",
        "Werkzeug==1.0.0",
        "wsproto==0.15.0",
        "yarl==1.4.2",
        "zipp==3.1.0",
        "nltk==3.4.5",
        "rake-nltk==1.0.4"
    ]

)




import datetime
import pandas as pd
from TrollHunter.twitter_crawler.User import User
from TrollHunter.twitter_crawler.tweet_obj import Tweet_obj
from TrollHunter.twitter_crawler.celeryapp import app
from TrollHunter.twitter_crawler.elastic import Elastic
from TrollHunter.twitter_crawler.twint import twint

HIDE_TWEET_OUPUT = True
elastic = Elastic()

"""
args:
    tweet:          set to 0 to avoid tweet (default: 1)
    follow:         set to 0 to avoid follow (default: 1)
    limit:          set the number of tweet to retrieve (Increments of 20, default: 100)
    follow_limit    set the number of following and followers to retrieve 
    since:          date selector for tweets (Example: 2017-12-27)
    until:          date selector for tweets (Example: 2017-12-27)
    retweet:        set to 1 to retrieve retweet (default: 0)
    search:         search terms
    
TODO: Retrieve tweet twitted to the user ?
"""
@app.task
def get_info_from_user(username, args):
    reset_data()
    user = User(username)

    get_info_user(user, args)
    elastic.store_users(user.info_df)

    if "tweet" not in args or int(args["tweet"]) == 1:
        get_tweet_from_user(user, args)
        elastic.store_tweets(user.tweets_df)

    if "follow" not in args or int(args["follow"]) == 1:
        get_follower_user(user, args)
        get_following_user(user, args)
        elastic.store_users(user.info_follow_df)
        elastic.store_interaction(user.interaction_df)

    return "user"


def get_follower_user(user, args):
    config = get_twint_config(args, user=user)
    config.Pandas_au = True
    config.User_full = False
    if "follow_limit" in args:
        config.Limit = int(args["follow_limit"])
    else:
        config.Limit = user.info_df.loc[0]["followers"]
    twint.run.Followers(config)
    user.set_follower_df(twint.output.panda.Follow_df)
    i = 1
    limit = config.Limit
    for username in user.follower_df.iloc[0]['followers']:
        follower = User(username)
        get_info_user(follower, args)
        user.set_follow_df(follower.info_df, follower.info_df.loc[0]['id'], user.info_df.loc[0]['id'])
        print("Processed ", i, "/", limit, " followers")
        i += 1


def get_following_user(user, args):
    config = get_twint_config(args, user=user)
    config.Pandas_au = True
    config.User_full = False
    if "follow_limit" in args:
        config.Limit = int(args["follow_limit"])
    else:
        config.Limit = user.info_df.loc[0]["following"]
    twint.run.Following(config)
    user.set_following_df(twint.output.panda.Follow_df)
    i = 1
    limit = config.Limit
    for username in user.following_df.iloc[0]['following']:
        following = User(username)
        get_info_user(following, args)
        user.set_follow_df(following.info_df, user.info_df.loc[0]['id'], following.info_df.loc[0]['id'])
        print("Processed ", i, "/", limit, " following")
        i += 1

def get_info_user(user, args):
    config = get_twint_config(args, user=user)
    config.User_full = True
    config.Profile_full = True
    config.Pandas = False
    config.Since = datetime.date.today().isoformat()
    # Need Lookup because bug with twint and flask
    twint.run.Search(config)
    twint.run.Lookup(config)
    user.set_user_id(config.User_id)
    user.set_info_to_df(twint.output.users_list[-1])


def get_tweet_from_user(user, args):
    config = get_twint_config(args, user=user)
    config.Profile = True
    config.Profile_full = True
    twint.output.tweets_list.clear()
    twint.run.Profile(config)
    user.set_tweet_df(twint.output.panda.Tweets_df)
    return twint.output.tweets_list


@app.task
def get_tweet_from_search(args):
    config = twint.Config()
    config.Username = None
    config.Store_object = True
    if not "search" in args:
        return " bad request"
    config.Search = args["search"]
    twint.output.tweets_list.clear()
    twint.run.Search(config)
    tweet_result = twint.output.tweets_list

    tweets_result_df = twint.output.panda.Tweets_df

    return format_tweet_to_html(tweet_result, "test")


@app.task
def get_origin_tweet(args):
    if "search" not in args:
        return " bad request"
    tweet = args["search"]

    config = get_twint_config(args)

    twint.output.tweets_list.clear()
    twint.run.Search(config)

    tweets = twint.output.tweets_list

    tweet_result = reversed([Tweet_obj(t) for t in tweets])
    origin = None

    for t in tweet_result:
        if t.check_equal(tweet):
            origin = t
            break

    res = []

    if origin:
        origin.pretty_print()
        res.append(origin)

    return format_tweet_to_html(res, "ORIGIN")


def get_twint_config(args, user=None):
    config = twint.Config()
    config.Hide_output = HIDE_TWEET_OUPUT
    if user is not None:
        config.Username = user.username
        config.User_id = user.user_id

    if "limit" in args:
        config.Limit = int(args["limit"])
    else:
        config.Limit = 100

    if "since" in args:
        config.Since = args["since"]
    if "until" in args:
        config.Until = args["until"]
    if "retweet" in args:
        config.Retweets = args["retweet"].lower() == "true"
    if "search" in args:
        config.Search = args["search"]

    config.Pandas = True
    config.Store_object = True
    return config


def format_tweet_to_html(tweets_list, word):
    ret = "<h1>tweet from {} </h1><br>".format(word)
    for tweet in tweets_list:
        ret += "date : {},  username : {}, name : {} like : {}, retweets count = {}, tweet : {} <br>".format(
            tweet.datestamp + ":" + tweet.timestamp,
            tweet.username,
            tweet.name,
            tweet.likes_count,
            tweet.retweets_count,
            tweet.tweet
        )
    return ret


def reset_data():
    twint.output.tweets_list.clear()
    twint.output.users_list.clear()



import requests
from bs4 import BeautifulSoup as Soup
import pandas as pd
import hashlib
from elasticsearch import Elasticsearch


# to check if an id (here the url) already exists in the ES
from TrollHunter.loggers import InfluxDBLog
from TrollHunter.news_crawler.database import update_sitemap, insert_sitemap


def check_id_in_es(es: Elasticsearch, index: str, id: str):
    return es.exists(index, id)


def if_influx_url(influxdb, url):
    if influxdb:
        InfluxDBLog().addEntry("Sitemap", "Crawling", 1, "URL", url )


def sort_loc(x):
    res = x.find("loc")
    return res.string if res else "None"


def reverse_check_exists(es: Elasticsearch, rangeOut: int, out, indexEs):
    out_cleaned = []
    ranges = rangeOut
    lenOut = len(out)
    if lenOut > 0:
        if (rangeOut > lenOut ):
            ranges = lenOut


        for i in range(lenOut - 1, lenOut - 1 - ranges, -1):
            value = out[i][1]
            if not check_id_in_es(es, "sitemaps", str(value)):
                out_cleaned.append(out[i])
                
            
        print("length sitemap: " +  str(len(out_cleaned)))
    return out_cleaned


# Pass the headers you want to retrieve from the xml such as ["loc", "lastmod"]

def parse_sitemap(sitemap, trust_levels, db_sitemaps, es: Elasticsearch, indexEs = "sitemaps", sort = None,  influxdb = False, range_check = 20):

    resp = requests.get(sitemap[0])
    new_list = ["Source"] + sitemap[2] + ["Trust level"]
    panda_out_total = pd.DataFrame([], columns=new_list)
    # we didn't get a valid response, bail
    if (200 != resp.status_code):
        return panda_out_total

    # BeautifulSoup to parse the document
    soup = Soup(resp.content, "xml")
    # find all the <url> tags in the document
    urls = soup.findAll('url')
    #Sorts the urls by the key specified
    if sort and urls:
        urls.sort(key=sort_loc)
    sitemaps = soup.findAll('sitemap')
    

    if not urls and not sitemaps:
        return panda_out_total

    sitemap_db = db_sitemaps.get(sitemap[0])
    url_headers = sitemap_db[2] if sitemap_db and sitemap_db[2] else sitemap[2]

    # Recursive call to the the function if sitemap contains sitemaps
    if sitemaps:
        for u in sitemaps:
            test = u.find('loc').string.strip('\n')
            if check_sitemap(u, db_sitemaps.get(test), url_headers, sitemap[3]):
                panda_recursive = parse_sitemap([test, None, url_headers, sitemap[3]], trust_levels, db_sitemaps, es, sort, influxdb = influxdb, range_check = range_check)
                panda_out_total = pd.concat([panda_out_total, panda_recursive], ignore_index=True)

    # storage for later...
    out = []

    # Creates a hash of the parent sitemap
    hash_sitemap = hashlib.md5(str(sitemap[0]).encode('utf-8')).hexdigest()

    # Extract the keys we want
    count = 1
    for u in urls:
        if count % range_check == 0 and not check_id_in_es(es, indexEs, u.find("loc").string):
            count = 1
        
        values = [hash_sitemap]
        #Log into influxdb
        if_influx_url(influxdb, sitemap[0])
        for head in url_headers:
            loc = None
            loc = u.find(head)
            if not loc:
                loc = "None"
            else:
                loc = loc.string
            values.append(loc)
        values.append(trust_levels[sitemap[3]])
        out.append(values)
        if count == range_check:
            out_cleaned = reverse_check_exists(es, range_check, out, indexEs)
            return build_panda_out(out_cleaned, panda_out_total, new_list)
        count += 1
    if len(out) < range_check:
        out_cleaned = reverse_check_exists(es, range_check, out, indexEs)
        return build_panda_out(out_cleaned, panda_out_total, new_list)
    else: 
        return build_panda_out(out, panda_out_total, new_list)


def build_panda_out(out, panda_out_total, new_list):
    # Create a dataframe
    panda_out = pd.DataFrame(out, columns=new_list)

    # If recursive then merge recursive dataframe
    if not panda_out_total.empty:
        panda_out = pd.concat([panda_out, panda_out_total], ignore_index=True)

    # returns the dataframe
    return panda_out


def check_sitemap(sitemap, data_sitemap, headers, id_trust):
    loc = sitemap.find('loc').string.strip("\n")
    lastmod = sitemap.find('lastmod').string if sitemap.find('lastmod') else None
    if data_sitemap:
        if lastmod and data_sitemap[1]:
            if data_sitemap[1] != lastmod:
                update_sitemap(loc, lastmod)
            else:
                return False
    else:
        insert_sitemap(loc, lastmod, headers, id_trust)
    return True




from flask import Flask, request

from TrollHunter.twitter_crawler.twint_api import request_twint

app = Flask(__name__)



@app.route('/tweets/<string:user>', methods=['GET'])
def user_tweet(user):
    request_twint.get_info_from_user.delay(user, request.args)
    return "200"


@app.route('/tweets/', methods=['GET'])
def search_tweet():
    return request_twint.get_tweet_from_search.delay(request.args)


@app.route('/tweets/origin/', methods=['GET'])
def origin_tweet():
        return request_twint.get_origin_tweet.delay(request.args)


def run():
    app.run()

if __name__ == '__main__':
    run()



import datetime

import logging as logme


class Datelock:
    _until = None
    _since = None
    _since_def_user = None


def convertToDateTime(string):
    dateTimeList = string.split()
    ListLength = len(dateTimeList)
    if ListLength == 2:
        return string
    if ListLength == 1:
        return string + " 00:00:00"
    else:
        return ""


def Set(Until, Since):
    logme.debug(__name__+':Set')
    d = Datelock()

    if Until:
        d._until = datetime.datetime.strptime(convertToDateTime(Until), "%Y-%m-%d %H:%M:%S")
    else:
        d._until = datetime.datetime.today()

    if Since:
        d._since = datetime.datetime.strptime(convertToDateTime(Since), "%Y-%m-%d %H:%M:%S")
        d._since_def_user = True
    else:
        d._since = datetime.datetime.strptime("2006-03-21 00:00:00", "%Y-%m-%d %H:%M:%S")
        d._since_def_user = False

    return d



import psycopg2


def connect_db():
    conn = psycopg2.connect(host='142.93.170.234', database='sitemap', user='trollhunter', password='trollhunter')
    return conn


def disconnect_db(conn, cur):
    if conn:
        cur.close()
        conn.close()


def get_sitemap_parent():
    conn = None
    cur = None
    try:
        conn = connect_db()
        cur = conn.cursor()
        query = """select * from sitemap where lastmod is null"""
        cur.execute(query)
        data = cur.fetchall()
        return data
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        disconnect_db(conn, cur)


def get_all_sitemap():
    conn = None
    cur = None
    try:
        conn = connect_db()
        cur = conn.cursor()
        query = """select * from sitemap"""
        cur.execute(query)
        data = cur.fetchall()
        return data
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        disconnect_db(conn, cur)


def get_sitemap(id):
    conn = None
    cur = None
    try:
        conn = connect_db()
        cur = conn.cursor()
        query = """select * from sitemap where url = %s"""
        cur.execute(query, (id,))
        data = cur.fetchone()
        return data
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        disconnect_db(conn, cur)


def update_sitemap(id, data):
    conn = None
    cur = None
    try:
        conn = connect_db()
        cur = conn.cursor()
        query = """update sitemap set lastmod = %s where url = %s"""
        cur.execute(query, (data, id))
        conn.commit()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        disconnect_db(conn, cur)


def insert_sitemap(loc, lasmod, url_headers, id_trust):
    conn = None
    cur = None
    try:
        conn = connect_db()
        cur = conn.cursor()
        query = """insert into sitemap values (%s, %s, %s, %s)"""
        cur.execute(query, (loc, lasmod, url_headers, id_trust))
        conn.commit()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        disconnect_db(conn, cur)


def get_trust_levels():
    conn = None
    cur = None
    try:
        conn = connect_db()
        cur = conn.cursor()
        query = """select * from trust_level"""
        cur.execute(query)
        data = cur.fetchall()
        return data
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        disconnect_db(conn, cur)
